<?xml version="1.0" encoding="UTF-8"?>
<!--
This file is part of the xframe software package
hosted at http://xframe.sourceforge.net

Copyright (c) 2003 Kurt Riede.
    
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xalan="http://xml.apache.org/xslt" xmlns:doc="http://xframe.sf.net/xsddoc/doc">
  <!--
    This transformation creates an index of a schema
  -->
  <xsl:output indent="yes" version="4.0" method="xml" encoding="ISO-8859-1" xalan:indent-amount="2" />
  <!--
    Include xframe utilities templates.
  -->
  <xsl:include href="util.xsl"/>
  <!--
    The file name of the schema to extract from.
  -->
  <xsl:param name="schemaLocation" select="undefined"/>
  <!--
    The element name to extract.
  -->
  <xsl:param name="type" select="undefined"/>
  <!--
    The name attribute of the element to extract.
  -->
  <xsl:param name="name" select="undefined"/>
  <!--
    Namespace prefix of target namespace
  -->
  <xsl:variable name="targetNamespacePrefix">
    <xsl:if test="/xs:schema/@targetNamespace">
      <xsl:value-of select="local-name(/xs:schema/namespace::*[normalize-space(.)=normalize-space(/xs:schema/@targetNamespace)])"/>
    </xsl:if>
  </xsl:variable>
  <!--
    Root template.
  -->
  <xsl:template match="/">
    <xsl:apply-templates select="*"/>
  </xsl:template>
  <!--
    Processing the schema tag, the root element of a schema.
    todo change to intermediate XML format
    todo also recurse included schemas
  -->
  <xsl:template match="xs:schema">
    <xsl:variable name="ns">
      <xsl:choose>
        <xsl:when test="normalize-space(@targetNamespace) = ''">
          <xsl:value-of select="string('noNamespace')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="/xs:schema/@targetNamespace"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <doc:overview namespace="{$ns}" href="./index.html">
      <xsl:call-template name="xsddocComment"/>
      <xsl:apply-templates select="." mode="overview"/>
    </doc:overview>
  </xsl:template>
  <!--
    process all schema components.
  -->
  <xsl:template match="xs:schema" mode="overview">
    <xsl:param name="ignore"/>
    <xsl:apply-templates select="xs:element | xs:attribute | xs:complexType | xs:simpleType | xs:group | xs:attributeGroup" mode="doc">
      <xsl:with-param name="ignore" select="$ignore"/>
    </xsl:apply-templates>
    <xsl:for-each select="xs:include | xs:redefine">
      <xsl:variable name="redefinedComponents">
        <xsl:call-template name="tostring">
          <xsl:with-param name="nodes" select="xs:element | xs:attribute | xs:complexType | xs:simpleType | xs:group | xs:attributeGroup"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:apply-templates select="xs:element | xs:attribute | xs:complexType | xs:simpleType | xs:group | xs:attributeGroup" mode="doc">
        <xsl:with-param name="ignore" select="$ignore"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="document(@schemaLocation)/xs:schema" mode="overview">
        <xsl:with-param name="ignore" select="$redefinedComponents"/>
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>
  <!--
    Create an index entry.
  -->
  <xsl:template match="*" mode="doc">
    <xsl:param name="ignore"/>
    <xsl:if test="not(contains($ignore, concat('|', local-name(), ':', @name, '|')))">
      <doc:component type="{local-name()}" name="{@name}" href="">
        <xsl:attribute name="href">
          <xsl:value-of select="concat(substring-after(name(), ':'), '/', @name, '.html')"/>
        </xsl:attribute>
      </doc:component>
    </xsl:if>
  </xsl:template>
  <!--
  -->
  <xsl:template name="tostring">
    <xsl:param name="nodes"/>
    <xsl:for-each select="$nodes">
      <xsl:value-of select="concat('|', local-name(), ':', @name, '|')"/>
    </xsl:for-each>
  </xsl:template>
</xsl:stylesheet>
